@@ -242,14 +242,15 @@ class CustomDataset(Dataset):
         
         if self.resample != None:
             if self.mode == 'train' :
+                if isinstance(x, np.ndarray):
+                    x = torch.tensor(x.copy())
                 if isinstance(y, np.ndarray):
                     y = torch.tensor(y.copy())
-                result = self.resample(torch.vstack((x, y.unsqueeze(0))), info_data)
-                x = result[:-1]
-                y = result[-1].squeeze(0)
+                x, y = self.resample(x, info_data, y)
+            
             
             else:
-                x = self.resample(x, info_data)
+                x, _ = self.resample(x, info_data)
                 
         if isinstance(x, np.ndarray):
             x = torch.tensor(x)
@@ -281,15 +282,41 @@ class CustomDataset(Dataset):
                 pickle_file.close()
 
 
-class CustomDatasetV2(CustomDataset):
-    def __init__(self, images_dir, label_dir = None, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = False, resample = None,
-                 patch_size = (128, 128, 128), transform = None, nifti = False):
+class CustomDatasetV2(Dataset):
+    def __init__(self, images_dir, label_dir = None, device = None, normalizer = None, mode = 'train', dim = '3d', correct_direction = False, resample = None, do_random_permute = False,
+                 patch_size = (128, 128, 128), transform = None, nifti = False, pad_val = -1024):
         
         self.nifti = nifti
         self.patch_size = np.array(patch_size)
         self.transform = transform
         self.batch_size = 1
-        super().__init__(device = device, normalizer = normalizer, mode = mode, dim = dim, correct_direction = correct_direction, resample = resample, root_dir = images_dir)
+        self.pad_val = pad_val
+        self.do_random_permute = do_random_permute
+        
+        self.root_dir = images_dir
+        self.image_name = []
+        self.info_name = []
+        self.device = device
+        self.normalizer = normalizer
+        self.mode = mode
+        self.dim = dim
+        self.resample = resample
+        self.correct_direction = correct_direction
+        if resample != None :
+            self.resample.device = device
+        if normalizer != None:
+            self.normalizer.device = device
+
+
+        for file_name in os.listdir(self.root_dir):            
+            if 'npz' in file_name:
+                self.image_name.append(file_name)
+            
+            elif 'pkl' in file_name:
+                self.info_name.append(file_name)
+                    
+        self.image_name = sorted(self.image_name)
+        self.info_name = sorted(self.info_name)
 
         if self.nifti:
             self.images_dir = images_dir
@@ -298,8 +325,10 @@ class CustomDatasetV2(CustomDataset):
             self.labels_list = os.listdir(label_dir)
 
     def __len__(self):
-        return super().__len__()
-    
+        if self.nifti:
+            return len(self.images_dir)
+        else:
+            return len(self.image_name)    
     
     def get_oversample_patch(self, images, seg):
         if not isinstance(images, np.ndarray):
@@ -311,8 +340,11 @@ class CustomDatasetV2(CustomDataset):
         non_zero_coords = np.array(np.where(seg != 0))
         shape_to_index = np.array(images[0].shape) - 1 
         
+
         oversample_high = np.clip(np.max(non_zero_coords, axis = 1) + 1, self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
         oversample_low = np.clip(np.min(non_zero_coords, axis = 1), self.patch_size/2 - 1, shape_to_index - self.patch_size/2)
+
+        print("\n oversample high : ", oversample_high, "oversample low : ", oversample_low)
         oversample_center = np.random.randint(low = oversample_low, high = oversample_high + 1)
 
         oversample_patch_upper_bound = oversample_center + (self.patch_size/2) + 1
@@ -326,6 +358,7 @@ class CustomDatasetV2(CustomDataset):
                                           oversample_patch_bound[1][0] : oversample_patch_bound[1][1],
                                           oversample_patch_bound[2][0] : oversample_patch_bound[2][1]]
         
+        print('oversmaple shape : ', oversample_patch_images.shape)
         return np.expand_dims(oversample_patch_images , axis = 0), np.expand_dims(oversample_patch_seg , axis = 0)
     
     def get_normal_patch(self, images, seg):
@@ -336,7 +369,12 @@ class CustomDatasetV2(CustomDataset):
             seg = seg.numpy()
 
         shape_to_index = np.array(images[0].shape) - 1 
-        normal_center = np.random.randint(low = self.patch_size/2 - 1, high = shape_to_index - (self.patch_size/2) + 1 )
+        low = self.patch_size/2 - 1
+        high = np.maximum(low, shape_to_index - (self.patch_size/2) + 1 )
+        low[high == low] -= 1
+        print("\n normal high : ", high, "normal low : ", low)
+
+        normal_center = np.random.randint(low = self.patch_size/2 - 1, high = high)
         normal_patch_upper_bound = normal_center + (self.patch_size/2) + 1
         normal_patch_lower_bound = normal_center - (self.patch_size/2 - 1)
         normal_patch_bound = np.concatenate((normal_patch_lower_bound.reshape(-1,1),normal_patch_upper_bound.reshape(-1,1)), axis = 1).astype(int)
@@ -347,6 +385,7 @@ class CustomDatasetV2(CustomDataset):
                                           normal_patch_bound[1][0] : normal_patch_bound[1][1],
                                           normal_patch_bound[2][0] : normal_patch_bound[2][1]]
         
+        print("normal shape : ", normal_patch_images.shape)
         return np.expand_dims(normal_patch_images , axis = 0), np.expand_dims(normal_patch_seg , axis = 0)
 
     
@@ -362,8 +401,8 @@ class CustomDatasetV2(CustomDataset):
             for i in range(len(odd_pad)):
                 even_pad[i*2] += odd_pad[i]
             even_pad = tuple(even_pad[::-1].astype(int).copy())
-            images = F.pad(images, even_pad, "constant", 0 )
-            seg = F.pad(seg, even_pad, "constant", 0 )
+            images = F.pad(images.float(), even_pad, "constant", self.pad_val)
+            seg = F.pad(seg.float(), even_pad, "constant", 0)
 
         return images, seg
 
@@ -378,6 +417,8 @@ class CustomDatasetV2(CustomDataset):
             if self.label_dir != None:
                 seg = sitk.ReadImage(os.path.join(self.label_dir, self.labels_list[idx]))
                 y = sitk.GetArrayFromImage(seg)
+            else:
+                y = None
         else:
             info_file = open(os.path.join(self.root_dir,self.info_name[idx]), 'rb')
             info_data = pickle.load(info_file)
@@ -386,6 +427,7 @@ class CustomDatasetV2(CustomDataset):
             x = data['x']
             y = data['y']
 
+
         if len(x.shape) == 3 and self.dim == '3d':
             #modality가 1개인 경우
             x = np.expand_dims(x, axis = 0)
@@ -402,43 +444,44 @@ class CustomDatasetV2(CustomDataset):
             if self.mode == 'train' :
                 y = set_direction(y, info_data['direction'])
 
-        if self.normalizer:
-            x = self.normalizer(x)
-        
-        
+
         if self.resample != None:
             if self.mode == 'train' :
                 if isinstance(x, np.ndarray):
                     x = torch.tensor(x.copy())
                 if isinstance(y, np.ndarray):
                     y = torch.tensor(y.copy())
-                result = self.resample(torch.vstack((x, y.unsqueeze(0))), info_data)
-                x = result[:-1]
-                y = result[-1].squeeze(0)
+                x, y = self.resample(x, info_data, y)
             
             else:
-                x = self.resample(x, info_data)
+                x, _ = self.resample(x, info_data)
         
         x, y = self.pad_image_seg(x, y)
-        if self.batch_size == 1 :
-            result_images, result_seg = self.get_normal_patch(x, y)
 
+        
+        if self.normalizer:
+            x = self.normalizer(x)
 
-        for i_b in range(self.batch_size//2):
-            oversample_images, oversample_seg = self.get_oversample_patch(x, y)
-            normal_images, normal_seg = self.get_normal_patch(x, y)
-
-            if i_b == 0:
-                result_images = np.concatenate((normal_images, oversample_images), axis = 0) #bs, n, x, y, z
-                result_seg = np.concatenate((normal_seg, oversample_seg), axis = 0) #bs, x, y, z
+        if self.do_random_permute:
+            prob = np.random.uniform()
+            if prob > 0.5:
+                selected_axis = np.array([-3,-2,-1])
+                np.random.shuffle(selected_axis)
             
-            else:
-                result_images = np.concatenate((result_images, oversample_images), axis = 0)
-                result_seg = np.concatenate((result_seg, oversample_seg), axis = 0)
-                result_images = np.concatenate((result_images, normal_images), axis = 0)
-                result_seg = np.concatenate((result_seg, normal_seg), axis = 0)
+                if isinstance(x, np.ndarray):
+                    x = torch.tensor(x)
+                x = x.permute(0,selected_axis[0],selected_axis[1],selected_axis[2])
+                if y != None:
+                    if isinstance(y, np.ndarray):
+                        y = torch.tensor(y)
+                    y = y.permute(selected_axis[0],selected_axis[1],selected_axis[2])
 
 
+        if np.random.uniform() > 0.5 :
+            result_images, result_seg = self.get_normal_patch(x, y)
+        else:
+            result_images, result_seg = self.get_oversample_patch(x, y)
+
         if isinstance(result_images, np.ndarray):
             result_images = torch.tensor(result_images)
         
