@@ -1,5 +1,7 @@
 from PIL.Image import MODES
 from ingradient_library.dataloads import *
+import matplotlib.pyplot as plt
+from tqdm import tqdm
 from torchvision.transforms import CenterCrop
 import torch
 import numpy as np
@@ -57,10 +59,8 @@ def get_coords_arr(images, patch_size, step_size):
     for i in range(3):
         step = 1+(image_shape[i] - patch_size[i])/(patch_size[i] * step_size[i])
         if step < 1:
-            print('step is zero')
             step = 1.0
         grid_list.append(torch.arange(step))
-    print(grid_list)
     grid_x, grid_y, grid_z = torch.meshgrid(*grid_list)
     grid_x = torch.round(grid_x * step_size[0] * patch_size[0]).long()
     grid_y = torch.round(grid_y * step_size[1] * patch_size[1]).long()
@@ -97,16 +97,46 @@ def change_value_from_coords(image, patch_coords, index, patch):
     for i in range(image.shape[0]):
         image[i][torch.meshgrid(*patch_coords[index])] += patch[i]
     return image
+"""
 
+class TTA(object):
+    def __init__(self, flip = [0,1,2], permute = [[1,0,2]]):
+        self.flip = flip
+        self.permute = permute
+
+    def transform(self, image):
+        images = [image]
+        transformed_log = [] # {'image':, 'flip':, 'permute': 
+
+        if self.permute != None:
+            for p in self.permute:
+                permutted_img = image.permute(p[0],p[1],p[2])
+                image.append(permutted_img)
+
+        for f in self.flip:
+            flipped_img = torch.flip(image, f)
+            images.append(flipped_img)
+            if self.permute != None:
+                for p in self.permute:
+                    permutted_img = flipped_img.permute(p[0],p[1],p[2])
+                    image.append(permutted_img)
+
+        return images
+        
+    def inverse_transform(self):
+
+"""
 
 class CC_Remover(object):
     # remain num => 남길 connected components label의 수
     # frequency order => connected component labeling 된 것 중 택할 빈도 수를 정함. 1은 보통 background 임.
-    def __init__(self, remain_num = 2, frequency_order = [2,3], distance = True):
+    def __init__(self, remain_num = 2, frequency_order = [2,3], distance = True, axis_z = 2, cc_label_counts = 1500, distance_weight = 20.0):
         self.remain_num = remain_num
         self.frequency_order = frequency_order
         self.distance = distance
-    
+        self.axis_z = axis_z
+        self.cc_label_counts = cc_label_counts
+        self.distance_weight = distance_weight
 
     def __call__(self, mask):
         conected_component_mask = cc3d.connected_components(mask, connectivity = 26)
@@ -114,11 +144,15 @@ class CC_Remover(object):
         result = torch.zeros(mask.shape)
 
         if self.distance != False:
-            remained_components_indices = np.argsort(count)[::-1][1:self.remain_num+2]
+            component_length = (count > self.cc_label_counts).sum()
+            remained_components_indices = np.argsort(count)[::-1][:component_length]
             center_coords = []
+            min_max_coords = []
             for r_i in remained_components_indices:
-                center_coords.append([np.where(conected_component_mask == r_i)[0].mean(), np.where(conected_component_mask == r_i)[1].mean(), np.where(conected_component_mask == r_i)[2].mean()])
+                min_max_coords.append([np.where(conected_component_mask == r_i)[0].max() - np.where(conected_component_mask == r_i)[0].min(), np.where(conected_component_mask == r_i)[1].max() - np.where(conected_component_mask == r_i)[1].min(), np.where(conected_component_mask == r_i)[2].max()- np.where(conected_component_mask == r_i)[2].min()])
+                center_coords.append([np.where(conected_component_mask == r_i)[self.axis_z].mean()])
             center_coords = np.array(center_coords)
+            min_max_coords = np.array(min_max_coords)
             n_clusters = center_coords.shape[0]
             distance = np.zeros((n_clusters, n_clusters))
 
@@ -127,7 +161,7 @@ class CC_Remover(object):
                     if i == j:
                         distance[i][j] = 1e+10
                     else:
-                        distance[i][j] = numpy.linalg.norm(center_coords[i]-center_coords[j])
+                        distance[i][j] = self.distance_weight * numpy.linalg.norm(center_coords[i]-center_coords[j]) + numpy.linalg.norm(min_max_coords[i]-min_max_coords[j])
 
             x, y = np.unravel_index(np.argmin(distance, axis=None), distance.shape)
             return torch.tensor((remained_components_indices[x] == conected_component_mask) + (remained_components_indices[y] == conected_component_mask))
@@ -149,7 +183,7 @@ class InferenceV2(object):
     deep_supervision : 모델에 deep_supervision이 적용 될 경우, True
     padding : patch size에 맞게 padding
     """
-    def __init__(self, dataset, model, patch_size, n_classes, mode = 'npy and nifti', step_size = [0.5, 0.5, 0.5], postprocessing = None, device = None, softmax_threshold = 0.99,
+    def __init__(self, dataset, model, patch_size, n_classes, mode = 'npy and nifti', step_size = [0.5, 0.5, 0.5], postprocessing = None, device = None, softmax_threshold = 0.99, tta = None,
                  batch_size = 1, gaussian_filter = True, deep_supervision = False, save_path = None, arg_max = False, padding = False, is_2d = False, sigma = 1e+3):
         self.dataset = dataset
         self.model = model.to(device)
@@ -172,6 +206,8 @@ class InferenceV2(object):
             self.gaussian_kernel = get_kernel(patch_size, sigma = sigma)
             self.gaussian_kernel = self.gaussian_kernel.to(device)
         
+        self.tta = tta
+        
     def valid_run(self):
         #전체 데이터에 대해 inference 시작.
         print('start valid run')
@@ -189,7 +225,13 @@ class InferenceV2(object):
             
             if isinstance(seg, np.ndarray):
                 seg = torch.from_numpy(seg)
+            
+            #if self.tta != None:
+            #    input_list = self.tta.transform(image)
+            #    output_list = []
+            #    for input in input_list:
             result = self.get_result(image, data_index, current_name)
+
             dice_score = []
             for i in range(1, self.n_classes):
                 temp_seg = (seg == i).long()
@@ -198,7 +240,7 @@ class InferenceV2(object):
                 union = temp_seg.sum() + temp_img.sum()
                 dice = 2 * intersection / union
                 dice_score.append(dice.numpy())
-            print(self.dataset.image_name[data_index], dice_score)
+            print("file_name : ", self.dataset.image_name[data_index], "dice : ", dice_score[0].item())
             final_score.append(dice_score)
         
         self.final_score = final_score
@@ -261,15 +303,15 @@ class InferenceV2(object):
         patch_bundle = torch.split(patch_bundle, self.bs)
         index_bundle = torch.split(index_bundle, self.bs)
         result = torch.zeros((self.n_classes, image.shape[-3], image.shape[-2], image.shape[-1]))
-        for mini_batch_patch, mini_batch_index in zip(patch_bundle, index_bundle):
+        for mini_batch_patch, mini_batch_index in tqdm(zip(patch_bundle, index_bundle)):
             with torch.no_grad():
                 output = self.model(mini_batch_patch.to(self.device).float())
-            if self.deep_supervision:
-                output = output[0]
-            output = torch.softmax(output, dim = 1) * self.gaussian_kernel.unsqueeze(0).unsqueeze(0)
-            output = output.cpu()
-            for batch_index in range(mini_batch_index.shape[0]):
-                result = change_value_from_coords(result, coords, mini_batch_index[batch_index], output[batch_index])
+                if self.deep_supervision:
+                    output = output[0]
+                output = torch.softmax(output, dim = 1) * self.gaussian_kernel.unsqueeze(0).unsqueeze(0)
+                output = output.detach().cpu()
+                for batch_index in range(mini_batch_index.shape[0]):
+                    result = change_value_from_coords(result, coords, mini_batch_index[batch_index], output[batch_index])
         
         if self.save_path != None:
             self.save(result, self.soft_path, data_index, current_name)
@@ -278,21 +320,21 @@ class InferenceV2(object):
             result = torch.argmax(result, 0)
         else:
             result = torch.softmax(result, dim = 0)
-            result[:, 1:] = (result[:, 1:] >= self.softmax_threshold)
-            result = torch.softmax(result, dim = 0)
+            result[1:] = result[1:] >= self.softmax_threshold
             result = torch.argmax(result, 0)
-            
+
         if self.save_path != None:
-            self.save(result, self.model_path, data_index, current_name, mode = 'seg', is_nifti = True)
+            argmax_result = torch.argmax(result, 0), 
+            self.save(argmax_result, self.model_path, data_index, current_name, mode = 'seg', is_nifti = True)
             self.save(image[0], self.image_path, data_index, current_name, mode = 'image', is_nifti = True)
 
-        if self.postprocessing != None:
-            mask = self.postprocessing((result != 0).detach().cpu().numpy())
-            result = mask * result
-        
-        self.save(result, self.model_path, data_index, current_name, mode = 'postprocessing', is_nifti = True)
+            if self.postprocessing != None:
+                mask = self.postprocessing((argmax_result != 0).detach().cpu().numpy())
+                argmax_result = mask * argmax_result
+                self.save(argmax_result, self.model_path, data_index, current_name, mode = 'postprocessing', is_nifti = True)
         
         return result
+    
 
 
 
