@@ -1,7 +1,16 @@
 from torch.optim.lr_scheduler import _LRScheduler
 
 class PolynomialLRDecay(_LRScheduler):
-    def __init__(self, optimizer, max_decay_steps, end_learning_rate=0.0001, power=0.9):
+    """Polynomial learning rate decay until step reach to max_decay_step
+    
+    Args:
+        optimizer (Optimizer): Wrapped optimizer.
+        max_decay_steps: after this step, we stop decreasing learning rate
+        end_learning_rate: scheduler stoping learning rate decay, value of learning rate must be this value
+        power: The power of the polynomial.
+    """
+    
+    def __init__(self, optimizer, max_decay_steps, end_learning_rate=1e-5, power=2.0):
         if max_decay_steps <= 1.:
             raise ValueError('max_decay_steps should be greater than 1.')
         self.max_decay_steps = max_decay_steps
