@@ -314,14 +314,14 @@ class Batch_Random_Permute(object):
         self.permute_index = permute_index
     
     def get_random_variables(self):
-        self.next_permute = np.random.shuffle(self.permute_index)
+        np.random.shuffle(self.permute_index)
 
     def __call__(self, images):
         if len(images.shape) == 5:
-            images = images.permute(0, 1, self.next_permute[0], self.next_permute[1], self.next_permute[2])
+            images = images.permute(0, 1, self.permute_index[0], self.permute_index[1], self.permute_index[2])
         
         elif len(images.shape) == 4:
-            images = images.permute(0, self.next_permute[0], self.next_permute[1], self.next_permute[2])
+            images = images.permute(0, self.permute_index[0], self.permute_index[1], self.permute_index[2])
 
         return images
 
