@@ -95,7 +95,7 @@ class Min_Max_Normalizer(object):
         images = 2 * ((images - self.min) / (self.max - self.min)) - 1
         return images
 
-class Fixed_Normalizer(object):
+class Clipping_Normalizer(object):
     def __init__(self, mean = -155.58, std = 324.70, min = -1024, max = 3025, device = None):
         self.mean = mean
         self.std = std
@@ -233,7 +233,7 @@ class Get_target_spacing(object):
 
 
 class ResamplingV2(object):
-    def __init__(self, anisotropy_axis_index = [], target_spacing = [1.0, 1.0, 1.0], random = False, shape_print = False,
+    def __init__(self, anisotropy_axis_index = [], target_spacing = [1.0, 1.0, 1.0], random = False, debug_shape = False,
                  device = None, spacing_order = [-3, -2, -1], do_check = False, check_threshold = 120):
         self.device = device
         self.anisotropy_axis_index = self.new_anisotropy_axis(anisotropy_axis_index)
@@ -242,7 +242,7 @@ class ResamplingV2(object):
         self.spacing_order = spacing_order
         self.do_check = do_check
         self.check_threshold = check_threshold
-        self.shape_print = shape_print
+        self.debug_shape = debug_shape
     
     def new_anisotropy_axis(self, array):
         for i in range(len(array)):
@@ -286,6 +286,12 @@ class ResamplingV2(object):
         if seg != None:
             seg = F.interpolate(seg.unsqueeze(0).unsqueeze(0), new_shape, mode = 'nearest').squeeze(0).squeeze(0)
 
-        if self.shape_print:
+        if self.debug_shape:
             print("img shape : ", img.shape, "seg shape : ", seg.shape)
+            img_shape = np.array(img.shape[-3:])
+            seg_shape = np.array(seg.shape[-3:])
+            
+            if np.any(img_shape != seg_shape):
+                print("Waring, Resampling was wrong")
+            
         return img, seg
