@@ -6,7 +6,7 @@ class Transform(object):
         self.probs = probs
         self.transforms = transforms
         self.type_img_transformer = [Batch_Gaussian_Noise, Batch_Gaussian_Blur_3D, Batch_Contrast, Batch_Brightness, Batch_GammaTransform, Batch_Low_Resolution]
-        self.type_img_seg_transformer = [Batch_Affine_3D, Batch_Mirroring]
+        self.type_img_seg_transformer = [Batch_Affine_3D, Batch_Mirroring, Batch_Random_Permute]
         self.type_resampling = [Resampling]
         self.img_transformer = []
         self.img_seg_transformer = []
@@ -43,8 +43,8 @@ class Transform(object):
                 if isinstance(self.img_seg_transformer[i], Batch_Affine_3D):
                     self.img_seg_transformer[i].get_matrices_and_coords(images)
                 
-                elif isinstance(self.img_seg_transformer[i], Batch_Mirroring):
-                    self.img_seg_transformer[i].get_mirror_axis()
+                else:
+                    self.img_seg_transformer[i].get_random_variables()
                 images = self.img_seg_transformer[i](images)
                 seg = self.img_seg_transformer[i](seg)
 
